# python
from urlparse import urlparse, urlunparse

# zope
from Globals import InitializeClass, Persistent
from AccessControl import ClassSecurityInfo
import ExtensionClass
from OFS.Traversable import Traversable
from Globals import Persistent
import Acquisition

# silva
from Products.Silva.SilvaObject import SilvaObject
from Products.Silva.interfaces import ISilvaObject
from Products.Silva import SilvaPermissions
from Products.SilvaMetadata.Exceptions import BindingError
from Products.Annotations.AnnotationTool import IAnnotable
# localizer
from Products.Localizer.LanguageManager import LanguageManager
from Products.Localizer.LocalAttributes import LocalAttributes, LocalAttribute
# XXX hmmm ... use LocalPropertyManager instead ?

# local
from interfaces import ILocalSilvaObject, ILocalRoot
from Errors import LocaleNotSupported, LocaleAlreadyExists
from LocalViewAttribute import LocalViewAttribute
from util import AcceptLanguageFake

METADATA_DELEGATE='_metadata_proxy'

class MetadataDelegate(Persistent, Acquisition.Implicit):
    __implements__ = (IAnnotable,)


# XXX why is ExtensionClass.Base needed here ???
class LocalSilvaObject(LanguageManager, LocalAttributes, \
                       Traversable, ExtensionClass.Base):
    """ abstract mixin class for all silva objects
    which support local variants

    basically this class implements managment of the available languages,
    a (possible) title handling and some hacks and patches
    """
    
    __implements__ = (ISilvaObject, ILocalSilvaObject)

    security = ClassSecurityInfo()

    edit = LocalViewAttribute('edit', 'tab_edit')

    _is_allowed_in_publication = 1

    def __init__(self, id, title, language):
        self._languages = (language, )
        # FIXME: cannot change default language later on :(
        self._default_language = language
        # FIXME: cannot set the title without context ...
        #self.set_title(title, lang=language)

    def _setupAfterAdd(self):
        try:
            bindings = self.service_metadata.getMetadata(self)
        except BindingError:
            # no metadata bindings: no local hacks
            return
        bindings.setObjectDelegator('get_metdata_proxy')
        id=METADATA_DELEGATE
        setattr(self, id, LocalAttribute(id))
        self._setLocalAttribute(id, MetadataDelegate())
        
        
    security.declarePrivate('get_metdata_proxy')
    def get_metdata_proxy(self):
        return self.getLocalAttribute(METADATA_DELEGATE)


    # implementation of ILocalSilvaObject
    # most methods are in LanguageManager anyway.
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_supported_languages')
    def get_supported_languages(self):
        """ return a list of supported languages,
        i.e. languages for which variants may be added
        """
        # XXX this needs to be overriden by containers, of course ...
        return self.get_container().get_available_languages()

    
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_supported_languages_sorted')
    def get_supported_languages_sorted(self):
        langs = self.get_supported_languages()
        all_langs = self.get_root().get_available_languages()
        return [ l for l in all_langs if l in langs ]

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'create_language_variant')
    def create_language_variant(self, new_language, old_language=None):
        """ create a new language variant, using the
        old language variant as a source
        (this means usually just to copy the corresponding information).
        If the old language is not given, the default language is choosen.
        """
        if not (new_language in self.get_supported_languages()):
            raise LocaleNotSupported, new_language
        if (old_language is not None) and \
               not (old_language in self.get_available_languages()):
            raise LocaleNotSupported, old_language
        if new_language in self.get_available_languages():
            raise LocaleAlreadyExists, new_language
            
        self.add_language(new_language)
        self._cretate_metadata_for_language(new_language, old_language)

    # little obscure helper to create the delegate carying the metadata bindings
    def _cretate_metadata_for_language(self, new_language, old_language):
        try:
            self.service_metadata.getMetadata(self)
        except BindingError:
            # no metadata - nothing to do
            return
        if old_language is None:
            old_language = self.get_selected_language()
        old_accept = self.REQUEST.other['AcceptLanguage']
        try:
            self.REQUEST.other['AcceptLanguage'] = dummy_accept = AcceptLanguageFake(old_language)

            title = self.get_title()
            dummy_accept.lang = new_language
            # XXX shouldn't we copy the stuff instead?
            self._setLocalAttribute(METADATA_DELEGATE, MetadataDelegate(), lang=new_language)
            self.set_title(title)
            self.sec_update_last_author_info()
        finally:
            self.REQUEST.other['AcceptLanguage'] = old_accept
        # XXX hmm, more ?
        
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_local_variant_url')
    def get_local_variant_url(self, lang, relative=None):
        """ return an (absolute) url to the variant of this
        object and the given language.
        It is not an error if the language does not exist at all
        (this is not our problem, but the one of the caller).
        """
        abs_url = LocalSilvaObject.inheritedAttribute('absolute_url')(self)
        def_lang = self.get_default_language()
        if lang == def_lang:
            return abs_url
        abs_url = list(urlparse(abs_url))
        path = abs_url[2]

        # FIXME: this only works if there is a Localizer in the current document root
        # i.e. this does not work for some VH settings, and not for a plain Zope !
        path = '/%s%s' % (lang,path)
        abs_url[2] = path

        abs_url = urlunparse(abs_url)
        return abs_url
        

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_public_languages')
    def get_public_languages(self):
        """ return a sequence of all languages
        for which a public variant exists
        """
        # by default, everything is public 
        return tuple(self.get_available_languages())


    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_public_languages_sorted')
    def get_public_languages_sorted(self):
        langs = self.get_public_languages()
        all_langs = self.get_root().get_available_languages()
        return [ l for l in all_langs if l in langs ]        

    security.declareProtected(SilvaPermissions.ReadSilvaContent,
                              'get_approved_languages')
    def get_approved_languages(self):
        """ return a sequence of all languages
        for which an approved variant exists
        """
        # by default, nothing is approved ... hmmm
        return ()


    def is_deletable(self):
        del_allowed = 1
        if self.get_public_languages():
            del_allowed = 0
        if self.get_approved_languages():
            del_allowed = 0
        return del_allowed
        
    #
    # overriding stuff from SilvaObject
    #

    def _setLocalAttribute(self, name, value, lang = None):
        if lang is None:
            lang = self.get_selected_language()
        # FIXME: should check if language is available
        # XXX copy & paste from LocalFolder
        # this would not be a problem, if we inherit from LocalPropertyManager
        # but I am somewhat afraid from inheriting something which implements __getattr__
        lang = lang.replace('-', '_')
        name = '%s_%s' % (name, lang)
        setattr(self, name, value)
        

    #
    # overriding stuff from LocalAttributes
    # instead of passing an exception if something is missing,
    # we return the "default" attribute
    #
    # XXX this could be solved better by mangling in
    # another accept_method, allowing to use the default as a fallback (?)
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'getLocalAttribute')
    def getLocalAttribute(self, name, lang=None):
        # XXX very ugly ugly hack: if the content object
        # is created with another language as the "default" (of the root)
        # we get a double attribute error :(
        super = LocalSilvaObject.inheritedAttribute('getLocalAttribute')
        try:
            return super(self, name, lang)
        except AttributeError:
            # self._log(-100,'attribute error',error=1)
            try:
                return super(self, name, \
                             lang = self.get_default_language())
            except AttributeError:
                # self._log(-100,'more attribute errors',error=1)
                # return super(self, name, self._languages[0])
                return super(self, name, self.get_available_languages()[0])
            

    # XXX debugging
    def _log(self, level=-100, message='log message missing', detail='', \
             error=None, reraise=None):
        import sys, zLOG
        if error:
            real_error = sys.exc_info()
        else:
            real_error = None
        zLOG.LOG('SilvaLocalContent', level, message, detail, \
                 real_error, reraise)

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'check_for_language_awareness')
    def check_for_language_awareness(self, object):
        # helper to check for language awareness
        # we cannot use the usual "implements_XXX"
        # as in general this would be acquired from the next SilvaLocalObject
        # and always return true, (except we would monkey patch it into SilvaObject)
        return ILocalSilvaObject.isImplementedBy(object)


    #
    # overriding from LanguageManager (via LocalAttributes)
    #
    def get_language_name(self, id=None):
        if id is None:
            id = self.get_default_language()
        return LocalSilvaObject.inheritedAttribute('get_language_name')(self, id)
    
    #
    # overriding even more
    #
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'absolute_url')
    def absolute_url(self, relative=0):
        " "
        # FIXME: looks a lot alike the get_local_variant_url
        abs_url = LocalSilvaObject.inheritedAttribute('absolute_url')(self,relative)
        sel_lang = self.get_selected_language()
        # XXX always uses the default languge from the root here ...
        def_lang = self.get_root().get_default_language()
        if (sel_lang is None) or (sel_lang == def_lang):
            return abs_url
        abs_url = list(urlparse(abs_url))
        path = abs_url[2]
        
        if relative:
            # path may be the empty string in certain VH settings
            # in this case we do _not_ want to create a trailing slash to the url
            if path:
                path = '%s/%s' % (sel_lang,path)
            else:
                path = sel_lang
        else:
            path = '/%s%s' % (sel_lang,path)
        
        abs_url[2] = path
        abs_url = urlunparse(abs_url)

        return abs_url


    def get_default_language(self):
        # XXX I want to have all objects to have the default language
        # of the root.
        # this is ugly, of course
        if ILocalRoot.isImplementedBy(self):
            return LocalSilvaObject.inheritedAttribute('get_default_language')(self)
        else:
            return self.get_root().get_default_language()


InitializeClass(LocalSilvaObject)
