import unittest

from OFS import SimpleItem
from Products.Silva.SilvaObject import SilvaObject

from Products.SilvaLocalContent.LocalSilvaObject import LocalSilvaObject
from Products.SilvaLocalContent.Errors import LocaleNotSupported, LocaleAlreadyExists
from Products.SilvaMetadata.Compatibility import registerTypeForMetadata
from Products.Localizer.LocalAttributes import LocalAttribute

from test_utils import assert_equal_elements

from SilvaLocalTestCase import SilvaLocalTestCase

class LocalTestContent(LocalSilvaObject, SilvaObject, SimpleItem.SimpleItem):
    " dummy class to be able to test the LocalSilvaObject mixin "
    # inherits from SimpleItem to have acquisition ...
    meta_type='Test Local Content'


registerTypeForMetadata(LocalTestContent.meta_type)

class LocalSilvaObjectTestCase(SilvaLocalTestCase):

    def afterSetUp(self):
        self.root.service_metadata.getTypeMapping()\
               .editMappings('',
                             [ {'type':LocalTestContent.meta_type,
                                'chain': 'silva-content, silva-extra' }, ]
                            )
        self.root._setObject('test',LocalTestContent('test',u'Title','en'))
        self.test = self.root.test
        self.test._setupAfterAdd()
        #id='_metadata_proxy'
        #setattr(self.test, id, LocalAttribute(id))
        #id='_metadata_proxy_en'
        #setattr(self.test, id, SimpleItem.SimpleItem(id))
        #id='_metadata_proxy_la'
        #setattr(self.test, id, SimpleItem.SimpleItem(id))

        bindings = self.root.service_metadata.getMetadata(self.test)
        bindings.setObjectDelegator('get_metdata_proxy')
        self.test.set_title(u'Title')

    def test_title(self):
        bindings = self.root.service_metadata.getMetadata(self.test)
        self.assertEquals(u'Title', bindings.get('silva-content','maintitle') )
        self.assertEquals(u'Title', self.test.get_title())

        self.acceptor.set_language('la')
        # XXX there is no variant for it: what should we do here ?
        # ... deciding to return the default instead ...
        self.assertEquals(u'Title', self.test.get_title())

        self.test.create_language_variant('la','en')
        self.assertEquals(u'Title', self.test.get_title())

        # FIXME without a supprted language, this should raise
        # a "language not supported" or the like ? instead it just sets
        # the default title, due to the lenitent implementation of "getLocalAttribute"
        self.test.set_title(u'Novus Titulus')
        self.assertEquals(u'Novus Titulus', self.test.get_title())

        self.acceptor.set_language('en')
        bindings = self.root.service_metadata.getMetadata(self.test)
        self.assertEquals(u'Title', bindings.get('silva-content','maintitle') )
        
        self.assertEquals(u'Title', self.test.get_title())

    def test_root_title(self):
        self.assertEquals(u'Root', self.root.get_title())
        self.acceptor.set_language('la')
        self.assertEquals(u'Root', self.root.get_title())


    def test_addLanguage(self):

        assert_equal_elements(['en', 'la'], self.root.get_available_languages())
        assert_equal_elements(['en', 'la'], self.root.get_supported_languages())

        assert_equal_elements(['en'], self.test.get_available_languages())
        assert_equal_elements(['en', 'la'], self.test.get_supported_languages())
        self.assertEquals('Title', self.test.get_title_editable())

        # copys the language from the default
        self.test.create_language_variant('la')
 
        assert_equal_elements(['en','la'], self.test.get_available_languages())

        self.test.set_title(u'New Title')
        self.assertEquals('New Title', self.test.get_title_editable())
        self.acceptor.set_language('la')
        self.assertEquals('Title', self.test.get_title_editable())
        self.test.set_title(u'Novum titulum')
        self.assertEquals(u'Novum titulum', self.test.get_title_or_id())
        
        self.acceptor.set_language('en')
        self.assertEquals(u'New Title', self.test.get_title_or_id())


    def test_addExistingLanguage(self):
        # a few possible failures 

        # does already exist
        try:
            self.test.create_language_variant('en')
        except LocaleAlreadyExists:
            pass
        else:
            self.fail('this test should have failed')
            
        # old language does not exist
        try:
            self.test.create_language_variant('la', old_language='fr')
        except LocaleNotSupported:
            pass
        else:
            self.fail('this test should have failed')
                    
        # language not supported by container
        try:
            self.test.create_language_variant('fr')
        except LocaleNotSupported:
            pass
        else:
            self.fail('this test should have failed')



    def test_public_languages(self):
        # for a plain local silva object, the published languages
        # are the same as the avialable langauges,
        # and the approved languages are empty

        assert_equal_elements(self.test.get_available_languages(),
                              self.test.get_public_languages())
        assert_equal_elements((),
                              self.test.get_approved_languages())

        # this remains true after adding a new variant ...
        self.test.create_language_variant('la')

        assert_equal_elements(self.test.get_available_languages(),
                              self.test.get_public_languages())
        assert_equal_elements((),
                              self.test.get_approved_languages())

        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(LocalSilvaObjectTestCase, 'test'))
    return suite        
