import unittest

from OFS import SimpleItem
from DateTime import DateTime

from Products.Silva.interfaces import IVersioning

from Products.SilvaLocalContent.interfaces import ILocalVersionedContent
from Products.SilvaLocalContent.LocalVersionedContent import LocalVersionedContent
from Products.SilvaLocalContent.LocalVersionedContent import LocaleVariant

from Products.SilvaLocalContent.Errors import LocaleNotSupported, LocaleAlreadyExists

from Testing.ZopeTestCase import ZopeTestCase
from SilvaLocalTestCase import SilvaLocalTestCase

from test_utils import assert_equal_elements

class SimpleVersion(SimpleItem.SimpleItem):
    def __init__(self, id):
        self.id = id
        self.data = None

    def set_data(self, data):
        self.data = data

    def get_data(self):
        return self.data
    
class LocalTestVersionedContent(LocalVersionedContent):
    " dummy class to be able to test the LocalVersionedContent "

    # XXX needed here, as not inherited from base class :(
    __implements__ = (ILocalVersionedContent,)

    _version_class = SimpleVersion
    def __init__(self, id, title, language):
        LocalTestVersionedContent.inheritedAttribute('__init__')(self, id, title, language)

class LocalVersionedContentTestCase(SilvaLocalTestCase):

    def afterSetUp(self):
        self.root._setObject('test',LocalTestVersionedContent('test',u'Title','en'))
        self.test = self.root.test
        self.test._setupAfterAdd()
        self.assert_(IVersioning.isImplementedBy(self.test))



    def test_approval(self):
        # a normal edit -> publish -> new edit -> publish -> close
        # workflow, not taking language variants into accout
        test = self.test

        editable = test.get_editable()
        self.assert_(editable is not None)
        editable.set_data('data')
        self.assertEquals('data', editable.get_data())
        self.assertEquals('data', test.get_previewable().get_data())

        test.set_unapproved_version_publication_datetime(DateTime()-1)
        test.approve_version()

        self.assert_(test.is_version_published())
        self.assert_(test.is_published())
        self.assert_(test.get_viewable() is not None)
        self.assertEquals('data', test.get_viewable().get_data())
        self.assert_(test.get_editable() is None)

        test.create_copy()
        
        self.assert_(test.is_published())
        editable = test.get_editable()
        self.assert_(editable is not None)
        self.assertEquals('data', editable.get_data())
        editable.set_data('data 2')
        self.assertEquals('data 2', editable.get_data())
        self.assertEquals('data 2', test.get_previewable().get_data())
        self.assertEquals('data', test.get_viewable().get_data())

        test.set_unapproved_version_publication_datetime(DateTime()-1)
        test.approve_version()

        self.assert_(test.is_published())
        self.assertEquals('data 2', test.get_viewable().get_data())

        test.close_version()
        self.failIf(test.is_published())
        self.assert_(test.get_viewable() is None)
        self.assert_(test.get_previewable() is not None)
        self.assertEquals('data 2', test.get_previewable().get_data())
        

    def test_variants(self):
        # test to create a new variant and publish it:
        # other language variant should not be affected
        test = self.test

        editable = test.get_editable()
        editable.set_data('data_en')
        self.assertEquals('data_en', test.get_editable().get_data())

        test.create_language_variant('la',old_language='en')
        self.acceptor.set_language('la')
        
        self.assertEquals('data_en', test.get_editable().get_data())
        test.get_editable().set_data('data_la')
        self.assertEquals('data_la', test.get_editable().get_data())
        test.set_unapproved_version_publication_datetime(DateTime()-1)
        test.approve_version()

        self.assert_(test.is_version_published())
        self.assert_(test.is_published())
        self.assert_(test.get_viewable() is not None)
        self.assertEquals('data_la', test.get_viewable().get_data())
        self.assert_(test.get_editable() is None)

        self.acceptor.set_language('en')
        self.failIf(test.is_published())
        self.failIf(test.get_viewable() is not None)
        self.failIf(test.get_editable() is None)
        self.assertEquals('data_en', test.get_previewable().get_data())


    def test_variant2(self):
        # test creating a new variant _after_ publishing the original
        test = self.test

        editable = test.get_editable()
        self.assert_(editable is not None)
        editable.set_data('data_en')
        self.assertEquals('data_en', editable.get_data())
        self.assertEquals('data_en', test.get_previewable().get_data())

        test.set_unapproved_version_publication_datetime(DateTime()-1)
        test.approve_version()
        self.assert_(test.is_published())

        test.create_language_variant('la',old_language='en')
        self.acceptor.set_language('la')
                     
        self.failIf(test.is_published())
        editable = test.get_editable()
        self.assert_(editable is not None)
        self.assertEquals('data_en', editable.get_data())
        editable.set_data('data_la')
        self.assertEquals('data_la', test.get_previewable().get_data())

        test.set_unapproved_version_publication_datetime(DateTime()-1)
        test.approve_version()
        self.assert_(test.is_published())
        self.assertEquals('data_la', test.get_viewable().get_data())

        # just to check: en-variant is unchanged
        self.acceptor.set_language('en')
        self.assertEquals('data_en', test.get_viewable().get_data())

        # now try to create a variant from a closed version ...
        self.root.add_language('fr')

        test.close_version()
        self.failIf(test.is_published())
        self.failIf(test.get_viewable() is not None)
        test.create_language_variant('fr',old_language='en')

        self.acceptor.set_language('fr')
        self.failIf(test.is_published()) 
        self.failIf(test.get_viewable() is not None)
        self.failIf(test.get_last_closed_version() is not None)
        self.failIf(test.get_editable() is None)
        self.assertEquals('data_en', test.get_editable().get_data())
        test.get_editable().set_data('data_fr')
        self.assertEquals('data_fr', test.get_previewable().get_data())

        # just to check: la-variant should be unchanged ...
        self.acceptor.set_language('la')
        self.assertEquals('data_la', test.get_viewable().get_data())        


    def test_differentVariants(self):
        # test to create several more variants
        # also checks, if the "old_language" parameter works properly
        # XXX this should be tested in LocalSilvaObject, shouldn't it ?
        # XXX partially this is already tested above :(

        test = self.test

        editable = test.get_editable()
        self.assert_(editable is not None)
        editable.set_data('data_en')

        self.acceptor.set_language('la')
        test.create_language_variant('la',old_language='en')
        self.assertEquals('data_en',test.get_editable().get_data())
        test.get_editable().set_data('data_la')

        self.acceptor.set_language('fr')
        try:
            test.create_language_variant('fr',old_language='en')
        except LocaleNotSupported:
            pass
        else:
            self.fail()
        self.root.add_language('fr')

        self.acceptor.set_language('en')
        test.create_language_variant('fr',old_language='la')
        self.assertEquals('data_en',test.get_editable().get_data())

        self.acceptor.set_language('fr')
        self.assertEquals('data_la',test.get_editable().get_data())
        test.get_editable().set_data('data_fr')
        
        self.acceptor.set_language('en')
        self.assertEquals('data_en',test.get_editable().get_data())
        self.acceptor.set_language('la')
        self.assertEquals('data_la',test.get_editable().get_data())
        self.acceptor.set_language('fr')
        self.assertEquals('data_fr',test.get_editable().get_data())
        

    def test_deleteAllowed(self):
        # a test that checks that delete is only allowed
        # if all language variants are closed
        # XXX this could be as well a test of the folder :(
        test = self.test

        test.create_language_variant('la',old_language='en')
        self.acceptor.set_language('la')
        test.set_unapproved_version_publication_datetime(DateTime()-1)
        test.approve_version()
        self.assert_(test.is_published())
        self.failIf(self.root.is_delete_allowed(test.getId()))

        self.acceptor.set_language('en')
        self.failIf(test.is_published())
        self.failIf(self.root.is_delete_allowed(test.getId()))
        self.root.action_delete( [test.getId() ] )
        self.assert_(hasattr(self.root, test.getId()) )
        self.assert_(test in self.root.get_ordered_publishables())

        self.acceptor.set_language('la')
        test.close_version()
        self.assert_(self.root.is_delete_allowed(test.getId()))

        # just add a new language, to check a stupid bug
        # where the "fr" variant has been checked, too ...
        self.root.add_language('fr')

        self.acceptor.set_language('en')
        self.assert_(self.root.is_delete_allowed(test.getId()))
        self.root.action_delete( [test.getId() ] )
        self.failIf(hasattr(self.root, test.getId()) )
        self.failIf(test in self.root.get_ordered_publishables())
                     


    def test_public_languages(self):
        test = self.test
        
        assert_equal_elements((),
                              test.get_public_languages())
        assert_equal_elements((),
                              test.get_approved_languages())

        # create a second variant
        test.create_language_variant('la', 'en')

        assert_equal_elements((),
                              test.get_public_languages())
        assert_equal_elements((),
                              test.get_approved_languages())

        test.set_unapproved_version_publication_datetime(DateTime()+100)
        test.approve_version()
        self.assert_(test.is_approved())
        
        assert_equal_elements((),
                              test.get_public_languages())
        assert_equal_elements(('en',),
                              test.get_approved_languages())

        self.acceptor.set_language('la')
        
        assert_equal_elements((),
                              test.get_public_languages())
        assert_equal_elements(('en',),
                              test.get_approved_languages())

        test.set_unapproved_version_publication_datetime(DateTime()-100)
        test.approve_version()
        self.assert_(test.is_published())

        assert_equal_elements(('la',),
                              test.get_public_languages())
        assert_equal_elements(('en',),
                              test.get_approved_languages())

        self.acceptor.set_language('en')
        test.set_approved_version_publication_datetime(DateTime()-100)
        self.assert_(test.is_published())
        
        assert_equal_elements(('la', 'en'),
                              test.get_public_languages())
        assert_equal_elements((),
                              test.get_approved_languages())

        test.close_version()
        assert_equal_elements(('la',),
                              test.get_public_languages())
        assert_equal_elements((),
                              test.get_approved_languages())        
    
    
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(LocalVersionedContentTestCase, 'test'))
    return suite        
